<?php
/**
 * Overrides carrier shipping with Table Rate Shipping
 *
 * Table Rate Shipping by Kahanit(https://www.kahanit.com/) is licensed under a
 * Creative Creative Commons Attribution-NoDerivatives 4.0 International License.
 * Based on a work at https://www.kahanit.com/.
 * Permissions beyond the scope of this license may be available at https://www.kahanit.com/.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-nd/4.0/.
 *
 * @author    Amit Sidhpura <amit@kahanit.com>
 * @copyright 2016 Kahanit
 * @license   http://creativecommons.org/licenses/by-nd/4.0/
 */

/**
 * Class TRSHelper
 *
 * @package Kahanit\TableRateShipping\Helper
 */
class TRSHelper
{
    public static function addslashes($str, $enclosed = false)
    {
        if (is_array($str)) {
            foreach ($str as &$item) {
                $item = TRSHelper::addslashes($item, $enclosed);
            }

            return $str;
        }

        $search = ["\\", "\0", "\n", "\r", "\x1a", "'", '"'];
        $replace = ["\\\\", "\\0", "\\n", "\\r", "\Z", "\'", '\"'];

        if ($enclosed) {
            $str = TRSHelper::addslashes($str);
        }

        return str_replace($search, $replace, $str);
    }

    public static function stripslashes($str)
    {
        if (is_array($str)) {
            foreach ($str as &$item) {
                $item = TRSHelper::stripslashes($item);
            }

            return $str;
        }

        $search = ["\\\\", "\\0", "\\n", "\\r", "\Z", "\'", '\"'];
        $replace = ["\\", "\0", "\n", "\r", "\x1a", "'", '"'];

        return str_replace($search, $replace, $str);
    }

    public static function checkZipCode($zip_code, $zip_code_format, $iso_code)
    {
        $iso_code_wildcard = preg_replace('/(.)/i', '([$1_%*]|)', $iso_code);

        $zip_regexp = '/^' . $zip_code_format . '$/ui';
        $zip_regexp = str_replace(' ', '( |)', $zip_regexp);
        $zip_regexp = str_replace('-', '(-|)', $zip_regexp);
        $zip_regexp = str_replace('N', '([0-9_%*]|)', $zip_regexp);
        $zip_regexp = str_replace('L', '([a-zA-Z_%*]|)', $zip_regexp);
        $zip_regexp = str_replace('C', $iso_code_wildcard, $zip_regexp);

        return (bool)preg_match($zip_regexp, $zip_code);
    }

    public static function installModuleTab($tabClass, $tabName, $idTabParent, $moduleName)
    {
        $tab = new Tab();

        foreach (Language::getLanguages() as $language) {
            $tab->name[$language['id_lang']] = $tabName;
        }

        $tab->class_name = $tabClass;
        $tab->module = $moduleName;
        $tab->id_parent = $idTabParent;

        if (!$tab->save()) {
            return false;
        }

        return true;
    }

    public static function uninstallModuleTab($tabClass)
    {
        $idTab = Tab::getIdFromClassName($tabClass);

        if ($idTab != 0) {
            $tab = new Tab($idTab);
            $tab->delete();

            return true;
        }

        return false;
    }
}
